
/* Chrysalide - Outil d'analyse de fichiers binaires
 * trigger.c - accès à des définitions Kaitai depuis ROST
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "trigger.h"


#include <string.h>


#include "browser.h"
#include "trigger-int.h"



/* ---------------------- ACCES ET DECLENCHEMENT D'UNE ANALYSE ---------------------- */


/* Initialise la classe des accès aux définitions Kaitai. */
static void g_kaitai_trigger_class_init(GKaitaiTriggerClass *);

/* Initialise un accès à une définition Kaitai. */
static void g_kaitai_trigger_init(GKaitaiTrigger *);

/* Supprime toutes les références externes. */
static void g_kaitai_trigger_dispose(GKaitaiTrigger *);

/* Procède à la libération totale de la mémoire. */
static void g_kaitai_trigger_finalize(GKaitaiTrigger *);



/* --------------------- IMPLEMENTATION DES FONCTIONS DE CLASSE --------------------- */


/* Indique le nom associé à une expression d'évaluation. */
static char *g_kaitai_trigger_get_name(const GKaitaiTrigger *);

/* Lance une résolution d'élément à solliciter. */
static bool g_kaitai_trigger_resolve(GKaitaiTrigger *, const char *, GScanContext *, GScanScope *, GScanRegisteredItem **);



/* ---------------------------------------------------------------------------------- */
/*                        ACCES ET DECLENCHEMENT D'UNE ANALYSE                        */
/* ---------------------------------------------------------------------------------- */


/* Indique le type défini pour un accès à une définition Kaitai pour ROST. */
G_DEFINE_TYPE(GKaitaiTrigger, g_kaitai_trigger, G_TYPE_SCAN_REGISTERED_ITEM);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des accès aux définitions Kaitai.       *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_trigger_class_init(GKaitaiTriggerClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */
    GScanRegisteredItemClass *registered;   /* Version de classe parente   */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_kaitai_trigger_dispose;
    object->finalize = (GObjectFinalizeFunc)g_kaitai_trigger_finalize;

    registered = G_SCAN_REGISTERED_ITEM_CLASS(klass);

    registered->get_name = (get_registered_item_name_fc)g_kaitai_trigger_get_name;
    registered->resolve = (resolve_registered_item_fc)g_kaitai_trigger_resolve;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : trigger = instance à initialiser.                            *
*                                                                             *
*  Description : Initialise un accès à une définition Kaitai.                 *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_trigger_init(GKaitaiTrigger *trigger)
{
    trigger->name = NULL;

    trigger->kstruct = NULL;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : trigger = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_trigger_dispose(GKaitaiTrigger *trigger)
{
    g_clear_object(&trigger->kstruct);

    G_OBJECT_CLASS(g_kaitai_trigger_parent_class)->dispose(G_OBJECT(trigger));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : trigger = instance d'objet GLib à traiter.                   *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_kaitai_trigger_finalize(GKaitaiTrigger *trigger)
{
    if (trigger->name != NULL)
        free(trigger->name);

    G_OBJECT_CLASS(g_kaitai_trigger_parent_class)->finalize(G_OBJECT(trigger));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : kstruct = définition Kaitai à manipuler avec du contenu.     *
*                                                                             *
*  Description : Crée un nouvel accès à une définition Kaitai à instancier.   *
*                                                                             *
*  Retour      : Instance mise en place ou NULL en cas d'échec.               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GScanRegisteredItem *g_kaitai_trigger_new(GKaitaiStruct *kstruct)
{
    GScanRegisteredItem *result;            /* Structure à retourner       */

    result = g_object_new(G_TYPE_KAITAI_TRIGGER, NULL);

    if (!g_kaitai_trigger_create(G_KAITAI_TRIGGER(result), kstruct))
        g_clear_object(&result);

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : trigger = lien vers une définition Kaitai à instancier.      *
*                kstruct = définition Kaitai à manipuler avec du contenu.     *
*                                                                             *
*  Description : Met en place un accès à une définition Kaitai pour ROST.     *
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool g_kaitai_trigger_create(GKaitaiTrigger *trigger, GKaitaiStruct *kstruct)
{
    bool result;                            /* Bilan à retourner           */

    result = true;

    trigger->kstruct = kstruct;
    g_object_ref(G_OBJECT(trigger->kstruct));

    return result;

}



/* ---------------------------------------------------------------------------------- */
/*                       IMPLEMENTATION DES FONCTIONS DE CLASSE                       */
/* ---------------------------------------------------------------------------------- */


/******************************************************************************
*                                                                             *
*  Paramètres  : trigger = élément d'appel à consulter.                       *
*                                                                             *
*  Description : Indique le nom associé à une expression d'évaluation.        *
*                                                                             *
*  Retour      : Désignation humaine de l'expression d'évaluation.            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static char *g_kaitai_trigger_get_name(const GKaitaiTrigger *trigger)
{
    char *result;                           /* Désignation à retourner     */
    GKaitaiMeta *meta;                      /* Eventuelles métadonnées     */
    const char *id;                         /* Identifiant de définition   */

    if (trigger->name != NULL)
        result = strdup(trigger->name);

    else
    {
        result = NULL;

        meta = g_kaitai_structure_get_meta(trigger->kstruct);
        if (meta == NULL) goto done;

        id = g_kaitai_meta_get_id(meta);
        if (id == NULL) goto done;

        result = strdup(id);

        g_object_unref(G_OBJECT(meta));

 done:

    }

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : item   = élément d'appel à consulter.                        *
*                target = désignation de l'objet d'appel à identifier.        *
*                ctx    = contexte de suivi de l'analyse courante.            *
*                scope  = portée courante des variables locales.              *
*                out    = zone d'enregistrement de la résolution opérée. [OUT]*
*                                                                             *
*  Description : Lance une résolution d'élément à solliciter.                 *
*                                                                             *
*  Retour      : Bilan de l'opération : false en cas d'erreur irrécupérable.  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static bool g_kaitai_trigger_resolve(GKaitaiTrigger *item, const char *target, GScanContext *ctx, GScanScope *scope, GScanRegisteredItem **out)
{
    bool result;                            /* Bilan à retourner           */
    GKaitaiBrowser *browser;                /* Navigateur pour résultats   */
    GBinContent *content;                   /* Contenu binaire à analyser  */
    GMatchRecord *record;                   /* Premier niveau de résultats */

    browser = g_object_get_data(G_OBJECT(item), "kaitai_browser");

    if (browser == NULL)
    {
        content = g_scan_context_get_content(ctx);

        record = g_kaitai_structure_parse(item->kstruct, content);

        g_object_unref(G_OBJECT(content));

        if (record != NULL)
        {
            browser = g_kaitai_browser_new(NULL, record);

            g_object_set_data_full(G_OBJECT(item), "kaitai_browser", browser, g_object_unref);

        }

    }

    if (browser == NULL)
        result = false;

    else
        result = g_scan_registered_item_resolve(G_SCAN_REGISTERED_ITEM(browser), target, ctx, scope, out);

    return result;

}
